<?php
///////////////////////////////////////////////////////////////////////////

require_once 'lib/vod/abstract_vod.php';
require_once 'lib/vod/movie.php';

///////////////////////////////////////////////////////////////////////////

class SundukVod extends AbstractVod
{
    private $session;
    protected $types;
    protected $years;
    protected $imdbs;
    protected $kinopoisks;
    protected $genres = array();

    public function get_session(){
        return $this->session;
    }
    
    public function ensure_movie_loaded($movie_id, &$plugin_cookies)
    {
        if (!isset($movie_id)) {
            throw new Exception('Movie ID is not set');
        }

        if ($this->is_failed_movie_id($movie_id)) {
            return null;
        }

        $movie = $this->get_cached_movie($movie_id);
        if ($movie === null || time() - $movie->loaded > 60*60 ) {
            $this->try_load_movie($movie_id, $plugin_cookies);
        }
    }

    public function __construct($session)
    {
        $this->session = $session;

        parent::__construct(true, true, false);
    }
    
    ///////////////////////////////////////////////////////////////////////

    public function get_vod_info(MediaURL $media_url, &$plugin_cookies)
    {
        $this->session->ensure_logged_in($plugin_cookies);

        return parent::get_vod_info($media_url, &$plugin_cookies);
    }

    public function try_load_movie($movie_id, &$plugin_cookies)
    {
        $movie = $this->session->api_vod_info($movie_id);

        $this->set_cached_movie($movie);
    }

    public function get_vod_stream_url($playback_url, &$plugin_cookies)
    {
        return $this->session->api_get_stream_url($playback_url);
    }

    public function get_buffering_ms()
    {
        $this->session->check_logged_in();

        $settings = $this->session->get_settings();
        return $settings->http_caching->value;
    }

    ///////////////////////////////////////////////////////////////////////
    // Favorites.

    protected function load_favorites(&$plugin_cookies)
    {
        $doc = $this->session->api_vod_favlist();

        if (!isset($doc->total))
            throw new Exception('Invalid data returned from server');

        $total = intval($doc->total);
        if ($total === 0)
        {
            $this->set_fav_movie_ids(array());
            return;
        }

        $fav_movie_ids = array();
        foreach ($doc->rows as $row)
        {
            $movie_id = $row->id;
            $movie_name = $row->name;
            $poster_url = 'http://' . SUNDUK::$SERVER . $row->poster;

            $fav_movie_ids[] = $movie_id;
            $this->set_cached_short_movie(
                new ShortMovie($movie_id, $movie_name, $poster_url));
        }
        $this->set_fav_movie_ids($fav_movie_ids);
        hd_print('The ' . count($fav_movie_ids) . ' favorite movies loaded.');
    }

    protected function do_add_favorite_movie($movie_id, &$plugin_cookies)
    {
        $this->session->api_vod_favadd($movie_id);
    }

    protected function do_remove_favorite_movie($movie_id, &$plugin_cookies)
    {
        $this->session->api_vod_favsub($movie_id);
    }

    ///////////////////////////////////////////////////////////////////////
    // Genres.

    protected function load_genres(&$plugin_cookies, $type = 'all')
    {
        $doc = $this->session->api_vod_genres($type);

        foreach ($doc->genres as $genre)
            $genres[$genre->id] = $genre->name;

        return $genres;
    }

    public function get_genre_icon_url($genre_id)
    {
        return $this->session->get_icon("vod_genre_$genre_id.png");
    }

    public function get_genre_media_url_str($genre_id, $type = 'all')
    {
        hd_print('SundukVod:get_genre_media_url_str $genre_id '.$genre_id);
        
        return SundukVodListScreen::get_media_url_str('last', array('genre' => $genre_id, 'type' => $type));
    }

    public function ensure_genres_loaded(&$plugin_cookies, $type = 'all')
    {
        if (isset($this->genres[$type]) && is_array($this->genres[$type])){
            return;
        }
        else{
            $this->genres[$type] = array();
        }
        
        $this->genres[$type] = $this->load_genres($plugin_cookies, $type);

        if ($this->genres === null)
            throw new Exception('Invalid VOD genres loaded');
    }

    public function get_genre_ids($type = 'all')
    {
        if (!is_array($this->genres[$type])){
            throw new Exception('VOD genres not loaded');
        }

        return array_keys($this->genres[$type]);
    }

    public function get_genre_caption($genre_id, $type = 'all')
    {
        if (!is_array($this->genres[$type])){
            throw new Exception('VOD genres not loaded');
        }

        return $this->genres[$type][$genre_id];
    }

    public function get_genres($type = 'all')
    {
        return $this->genres[$type];
    }
    
    ///////////////////////////////////////////////////////////////////////
    // Types.

    protected function load_types(&$plugin_cookies)
    {
        $doc = $this->session->api_vod_types();

        foreach ($doc->types as $type)
            $types[$type->id] = $type->name;

        return $types;
    }

    public function get_type_icon_url($type_id)
    {
        return $this->session->get_icon("vod_type_$type_id.png");
    }

    public function get_type_media_url_str($type_id)
    {
        hd_print('SundukVod:get_type_media_url_str $type_id '.$type_id);
        
        return SundukVodListScreen::get_media_url_str('last', array('type' => $type_id));
    }

    public function ensure_types_loaded(&$plugin_cookies)
    {
        if ($this->types !== null ){
            return;
        }
        else{
            $this->types = array();
        }
        $this->types = $this->load_types($plugin_cookies);

        if ($this->types === null)
            throw new Exception('Invalid VOD types loaded');
    }

    public function get_type_ids()
    {
        if ($this->types === null){
            throw new Exception('VOD types not loaded');
        }

        return array_keys($this->types);
    }

    public function get_type_caption($type_id)
    {
        if ($this->types === null){
            throw new Exception('VOD types not loaded');
        }

        return $this->types[$type_id];
    }

    public function get_types()
    {
        return $this->types;
    }
    
    ///////////////////////////////////////////////////////////////////////
    // Years.

    protected function load_years(&$plugin_cookies)
    {
        $doc = $this->session->api_vod_years();

        foreach ($doc->years as $year)
            $years[$year->id] = $year->name;

        return $years;
    }

    public function get_year_icon_url($year_id)
    {
        return $this->session->get_icon("vod_year_$year_id.png");
    }

    public function get_year_media_url_str($year_id)
    {
        hd_print('SundukVod:get_year_media_url_str $year_id '.$year_id);
        
        return SundukVodListScreen::get_media_url_str('last', array('year' => $year_id));
    }

    public function ensure_years_loaded(&$plugin_cookies)
    {
        if ($this->years !== null ){
            return;
        }
        else{
            $this->years = array();
        }
        
        $this->years = $this->load_years($plugin_cookies);

        if ($this->years === null)
            throw new Exception('Invalid VOD years loaded');
    }

    public function get_year_ids()
    {
        if ($this->years === null){
            throw new Exception('VOD years not loaded');
        }

        return array_keys($this->years);
    }

    public function get_year_caption($year_id)
    {
        if ($this->years === null){
            throw new Exception('VOD years not loaded');
        }

        return $this->years[$year_id];
    }

    public function get_years()
    {
        return $this->years;
    }
    
    ///////////////////////////////////////////////////////////////////////
    // IMDBs.

    protected function load_imdbs(&$plugin_cookies)
    {
        $doc = $this->session->api_vod_imdbs();

        foreach ($doc->imdb as $imdb)
            $imdbs[$imdb->id] = $imdb->id;

        return $imdbs;
    }

    public function get_imdb_icon_url($imdb_id)
    {
        return $this->session->get_icon("vod_imdb_$imdb_id.png");
    }

    public function get_imdb_media_url_str($imdb_id)
    {
        hd_print('SundukVod:get_imdb_media_url_str $imdb_id '.$imdb_id);
        
        return SundukVodListScreen::get_media_url_str('last', array('imdb' => $imdb_id));
    }

    public function ensure_imdbs_loaded(&$plugin_cookies)
    {
        if ($this->imdbs !== null ){
            return;
        }
        else{
            $this->imdbs = array();
        }
        
        $this->imdbs = $this->load_imdbs($plugin_cookies);

        if ($this->imdbs === null)
            throw new Exception('Invalid VOD imdbs loaded');
    }

    public function get_imdb_ids()
    {
        if ($this->imdbs === null){
            throw new Exception('VOD imdbs not loaded');
        }

        return array_keys($this->imdbs);
    }

    public function get_imdb_caption($imdb_id)
    {
        if ($this->imdbs === null){
            throw new Exception('VOD imdbs not loaded');
        }

        return $this->imdbs[$imdb_id];
    }

    public function get_imdbs()
    {
        return $this->imdbs;
    }
    
    ///////////////////////////////////////////////////////////////////////
    // Kinopoisks.

    protected function load_kinopoisks(&$plugin_cookies)
    {
        $doc = $this->session->api_vod_kinopoisks();

        foreach ($doc->kinopoisk as $kinopoisk)
            $kinopoisks[$kinopoisk->id] = $kinopoisk->id;

        return $kinopoisks;
    }

    public function get_kinopoisk_icon_url($kinopoisk_id)
    {
        return $this->session->get_icon("vod_kinopoisk_$kinopoisk_id.png");
    }

    public function get_kinopoisk_media_url_str($kinopoisk_id)
    {
        hd_print('SundukVod:get_kinopoisk_media_url_str $kinopoisk_id '.$kinopoisk_id);
        
        return SundukVodListScreen::get_media_url_str('last', array('kinopoisk' => $kinopoisk_id));
    }

    public function ensure_kinopoisks_loaded(&$plugin_cookies)
    {
        if ($this->kinopoisks !== null ){
            return;
        }
        else{
            $this->kinopoisks = array();
        }
        
        $this->kinopoisks = $this->load_kinopoisks($plugin_cookies);

        if ($this->kinopoisks === null)
            throw new Exception('Invalid VOD kinopoisks loaded');
    }

    public function get_kinopoisk_ids()
    {
        if ($this->kinopoisks === null){
            throw new Exception('VOD kinopoisks not loaded');
        }

        return array_keys($this->kinopoisks);
    }

    public function get_kinopoisk_caption($kinopoisk_id)
    {
        if ($this->kinopoisks === null){
            throw new Exception('VOD kinopoisks not loaded');
        }

        return $this->kinopoisks[$kinopoisk_id];
    }

    public function get_kinopoisks()
    {
        return $this->kinopoisks;
    }
    
    ///////////////////////////////////////////////////////////////////////
    // Search.

    public function get_search_media_url_str($pattern)
    {
        return SundukVodListScreen::get_media_url_str('last', array('pattern' => $pattern));
    }

    ///////////////////////////////////////////////////////////////////////
    // Folder views.

    public function get_vod_genres_folder_views()
    {
        $mov_genre_default_icon_url =
            $this->session->get_icon('mov_genre_default.png');

        return array(
            array
            (
                PluginRegularFolderView::async_icon_loading => false,

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 3,
                    ViewParams::num_rows => 10,
#                    ViewParams::content_box_padding_bottom => 110,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_padding_top => 0,
                    ViewItemParams::item_padding_bottom => 0,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::item_layout => HALIGN_LEFT,
                    ViewItemParams::item_caption_width => 365,
                    ViewItemParams::icon_dx => 65,
                ),

                PluginRegularFolderView::not_loaded_view_item_params => array
                (
                    ViewItemParams::icon_path => $mov_genre_default_icon_url,
                ),
            ));
    }

    public function get_vod_list_folder_views()
    {
        $mov_unset_url = $this->session->get_icon('mov_unset.png');

        return array(
            array
            (
                PluginRegularFolderView::async_icon_loading => true,

                PluginRegularFolderView::view_params => array
                (
                    ViewParams::num_cols => 6,
                    ViewParams::num_rows => 3,
                    ViewParams::icon_selection_box_width => 150,
                    ViewParams::icon_selection_box_height => 222,
                    ViewParams::paint_details => true,
                    ViewParams::zoom_detailed_icon => true,
                ),

                PluginRegularFolderView::base_view_item_params => array
                (
                    ViewItemParams::item_padding_top => 0,
                    ViewItemParams::item_padding_bottom => 0,
                    ViewItemParams::icon_valign => VALIGN_CENTER,
                    ViewItemParams::item_paint_caption => false,
                    ViewItemParams::icon_scale_factor => 1.0,
                    ViewItemParams::icon_sel_scale_factor => 1.2,
                ),

                PluginRegularFolderView::not_loaded_view_item_params => array
                (
                    ViewItemParams::icon_path => $mov_unset_url,
                    ViewItemParams::item_detailed_icon_path => 'missing://',
                ),
            ));
    }

    public function get_archive(MediaURL $media_url){
        return $this->session->get_archive();
    }

    public function folder_entered(MediaURL $media_url, &$plugin_cookies)
    {
        $this->session->ensure_logged_in($plugin_cookies);
    }
}

///////////////////////////////////////////////////////////////////////////
?>
